package org.act.manage.controls;
import java.io.Serializable;
import java.util.HashMap;

public class TreeControl implements Serializable {
        
    public TreeControl() {
        super();
        setRoot(null);
    }
    
    public TreeControl(TreeControlNode root) {
        super();
        setRoot(root);
    }
        
    protected HashMap registry = new HashMap();
    
    protected TreeControlNode selected = null;
        
    protected TreeControlNode root = null;
    public TreeControlNode getRoot() {
        return (this.root);
    }
    protected void setRoot(TreeControlNode root) {
        if (this.root != null)
            removeNode(this.root);
        if (root != null)
            addNode(root);
        root.setLast(true);
        this.root = root;
    }
    
    public int getWidth() {
        if (root == null)
            return (0);
        else
            return (getWidth(root));
    }
     
    public TreeControlNode findNode(String name) {
        synchronized (registry) {
            return ((TreeControlNode) registry.get(name));
        }
    }
 
    public void selectNode(String name) {
        if (selected != null) {
            selected.setSelected(false);
            selected = null;
        }
        selected = findNode(name);
        if (selected != null)
            selected.setSelected(true);
    }
      
    void addNode(TreeControlNode node) throws IllegalArgumentException {
        synchronized (registry) {
            String name = node.getName();
            if (registry.containsKey(name))
                throw new IllegalArgumentException("Name '" + name +
                                                   "' is not unique");
            node.setTree(this);
            registry.put(name, node);
        }
    }
    
    int getWidth(TreeControlNode node) {
        int width = node.getWidth();
        if (!node.isExpanded())
            return (width);
        TreeControlNode children[] = node.findChildren();
        for (int i = 0; i < children.length; i++) {
            int current = getWidth(children[i]);
            if (current > width)
                width = current;
        }
        return (width);
    }
   
    public void removeNode(TreeControlNode node) {
        synchronized (registry) {
            TreeControlNode children[] = node.findChildren();
            for (int i = 0; i < children.length; i++)
                removeNode(children[i]);
            TreeControlNode parent = node.getParent();
            if (parent != null) {
                parent.removeChild(node);
            }
            node.setParent(null);
            node.setTree(null);
            if (node == this.root) {
                this.root = null;
            }
            registry.remove(node.getName());
        }
    }
}
