
package org.act.manage.util;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Vector;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Attr;
import org.w3c.dom.CharacterData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class XMLUtils {
		
	public static DocumentBuilder getDOMBuilder()
			throws ParserConfigurationException {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		return dbf.newDocumentBuilder();
	}
	
	public static Transformer getTransformer()
			throws TransformerConfigurationException {
		TransformerFactory tf = TransformerFactory.newInstance();
		return tf.newTransformer();
	}
		
	public static Document createDocument() throws ParserConfigurationException {
		return XMLUtils.getDOMBuilder().newDocument();
	}
		
	public static Element getFirstChildElement(Node node) {
		for (Node tempNode = node.getFirstChild(); tempNode != null; tempNode = tempNode
				.getNextSibling()) {
			if (tempNode.getNodeType() == Node.ELEMENT_NODE) 				return (Element) tempNode;
		}
				return null;
	}
	
	public static Element getNextSiblingElement(Node node) {
		for (Node tempNode = node.getNextSibling(); tempNode != null; tempNode = tempNode
				.getNextSibling()) {
			if (tempNode.getNodeType() == Node.ELEMENT_NODE)
				return (Element) tempNode;
		}
		return null;
	}
	
	public static Element getLastChildElement(Node node) {
		for (Node tempNode = node.getLastChild(); tempNode != null; tempNode = tempNode
				.getPreviousSibling()) {
			if (tempNode.getNodeType() == Node.ELEMENT_NODE)
				return (Element) tempNode;
		}
		return null;
	}
	
	public static Element getPreviousSiblingElement(Node node) {
		for (Node tempNode = node.getPreviousSibling(); tempNode != null; tempNode = tempNode
				.getPreviousSibling()) {
			if (tempNode.getNodeType() == Node.ELEMENT_NODE)
				return (Element) tempNode;
		}
		return null;
	}
	
	public static Vector getAllChildElements(Node node) {
		Vector v = new Vector();
		Node child = XMLUtils.getFirstChildElement(node);
		while (child != null) {
			v.add(child);
			child = XMLUtils.getNextSiblingElement(child);
		}
		return v;
	}
	
	public static Vector getChildElementsByNS(Node node, String uri,
			String localName) {
		Vector childElements = XMLUtils.getAllChildElements(node);
		Vector v = new Vector();
		for (int i = 0; i < childElements.size(); i++) {
			Element child = (Element) childElements.get(i);
			String ln = child.getLocalName();
			String u = child.getNamespaceURI();
			if (uri == null) {
				if (ln.equals(localName))
					v.add(child);
			} else {
				if (ln.equals(localName) && u.equals(uri))
					v.add(child);
			}
		}
		return v;
	}
	
	public static String getTextValueOfElement(Element el) {
		NodeList nl = el.getChildNodes();
		Node node = null;
		for (int i = 0; i < nl.getLength(); i++) {
			node = nl.item(i);
			if ((node instanceof Text) && (node.getNodeValue().length() > 0))
				return node.getNodeValue();
		}
		return null;
	}
			
	public static String getLocalNameFromQName(String nodeName) {
		int index = nodeName.lastIndexOf(':');
		if (index == -1)
			return nodeName;
		else
			return nodeName.substring(index + 1, nodeName.length());
	}
	
	public static String getPrefixFromQName(String nodeName) {
		int index = nodeName.lastIndexOf(':');
		if (index == -1)
			return null;
		else
			return nodeName.substring(0, index);
	}
	
	public static String getLocalPartFromQName(String nodeName) {
		int index = nodeName.lastIndexOf(':');
		if (index == -1)
			return nodeName;
		else
			return nodeName.substring(index + 1);
	}
	
	public static String getNamespaceByPrefix(Element element, String prefix) {
		if (prefix == null)
			return null;
		String attrName;
		if (prefix.equals(""))
			attrName = "xmlns";
		else
			attrName = "xmlns:" + prefix;
		while (element != null) {
			NamedNodeMap nnm = element.getAttributes();
			Node nodeNamespace = nnm.getNamedItem(attrName);
			if (nodeNamespace != null)
				return nodeNamespace.getNodeValue();
			Node parent = element.getParentNode();
			if (parent.getNodeType() == Node.ELEMENT_NODE)
				element = (Element) parent;
			else
				break;
		}
		return null;
	}
		
	public static String nodeToPrettyString(Node node) {
		return nodeToPrettyString(node, "\t");
	}
	
	public static String nodeToPrettyString(Node node, String indentString) {
		StringWriter sw = new StringWriter();
		try {
			printPretty(node, sw, 0, indentString);
			return sw.toString();
		} catch (IOException ex) {
			return null;
		}
	}
	
	public static String nodeToPackedString(Node node) {
		StringWriter sw = new StringWriter();
		try {
			printPacked(node, sw);
			return sw.toString();
		} catch (IOException ex) {
			return null;
		}
	}
	
	public static String nodeToString(Node node) {
		StringWriter sw = new StringWriter();
		try {
			print(node, sw);
			return sw.toString();
		} catch (IOException ex) {
			return null;
		}
	}
	
	public static void nodeToString(Node node, Writer writer)
			throws IOException {
		print(node, writer);
	}
	
	private static void print(Node node, Writer out) throws IOException {
		if (node == null)
			return;
		switch (node.getNodeType()) {
		case Node.DOCUMENT_NODE: {
			out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			out.write(JavaUtils.LS);
			NodeList children = node.getChildNodes();
			if (children != null) {
				int numChildren = children.getLength();
				for (int i = 0; i < numChildren; i++)
					print(children.item(i), out); 			}
			break;
		}
		case Node.ELEMENT_NODE: {
			out.write('<' + node.getNodeName());
						NamedNodeMap attrs = node.getAttributes();
			int len = (attrs != null) ? attrs.getLength() : 0;
			for (int i = 0; i < len; i++) {
				Attr attr = (Attr) attrs.item(i);
				out.write(' ' + attr.getNodeName() + "=\""
						+ normalize(attr.getValue()) + '\"');
			}
						NodeList children = node.getChildNodes();
			int numChildren = (children != null) ? children.getLength() : 0;
			if (numChildren > 0) {
				out.write('>');
				for (int i = 0; i < numChildren; i++)
					print(children.item(i), out); 				out.write("</");
				out.write(node.getNodeName());
				out.write('>');
			} else
				out.write("/>");
			break;
		}
		case Node.ENTITY_REFERENCE_NODE: {
			out.write('&');
			out.write(node.getNodeName());
			out.write(';');
			break;
		}
		case Node.CDATA_SECTION_NODE: {
			out.write("<![CDATA[");
			out.write(node.getNodeValue());
			out.write("]]>");
			break;
		}
		case Node.TEXT_NODE: {
			out.write(normalize(node.getNodeValue())); 			break;
		}
		case Node.COMMENT_NODE: {
			out.write("<!--");
			out.write(node.getNodeValue());
			out.write("-->");
			break;
		}
		case Node.PROCESSING_INSTRUCTION_NODE: {
			out.write("<?");
			out.write(node.getNodeName());
			String data = node.getNodeValue();
			if (data != null && data.length() > 0) {
				out.write(' ');
				out.write(data);
			}
			out.write("?>");
			out.write(JavaUtils.LS);
			break;
		}
		}
	} 	
	private static void printPretty(Node node, Writer out, int level,
			String indentString) throws IOException {
		if (node == null)
			return;
		writeIndent(out, level, indentString);
		switch (node.getNodeType()) {
		case Node.DOCUMENT_NODE: {
			out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			out.write(JavaUtils.LS);
			NodeList children = node.getChildNodes();
			if (children != null) {
				int numChildren = children.getLength();
				for (int i = 0; i < numChildren; i++) {
										if (children.item(i).getNodeType() != Node.TEXT_NODE
							|| (!children.item(i).getNodeValue().trim().equals(
									""))) {
						printPretty(children.item(i), out, level, indentString); 					}
				}
			}
			break;
		}
		case Node.ELEMENT_NODE: {
			out.write('<' + node.getNodeName());
						NamedNodeMap attrs = node.getAttributes();
			int len = (attrs != null) ? attrs.getLength() : 0;
			for (int i = 0; i < len; i++) {
				Attr attr = (Attr) attrs.item(i);
				out.write(' ' + attr.getNodeName() + "=\""
						+ normalize(attr.getValue()) + '\"');
			}
						NodeList children = node.getChildNodes();
			int numChildren = (children != null) ? children.getLength() : 0;
			if (numChildren > 0) {
				out.write('>');
				if (numChildren == 1
						&& children.item(0).getNodeType() == Node.TEXT_NODE) {
										out
							.write(normalize(children.item(0).getNodeValue())
									.trim());
				} else {
					out.write(JavaUtils.LS);
					for (int i = 0; i < numChildren; i++) {
						if (children.item(i).getNodeType() != Node.TEXT_NODE
								|| (!children.item(i).getNodeValue().trim()
										.equals(""))) {
							printPretty(children.item(i), out, level + 1,
									indentString); 						}
					}
					writeIndent(out, level, indentString);
				}
				out.write("</");
				out.write(node.getNodeName());
				out.write('>');
			} else {
				out.write("/>");
			}
			out.write(JavaUtils.LS);
			break;
		}
		case Node.ENTITY_REFERENCE_NODE: {
			out.write('&');
			out.write(node.getNodeName());
			out.write(';');
			out.write(JavaUtils.LS);
			break;
		}
		case Node.CDATA_SECTION_NODE: {
			out.write("<![CDATA[");
			out.write(node.getNodeValue());
			out.write("]]>");
			out.write(JavaUtils.LS);
			break;
		}
		case Node.TEXT_NODE: {
			out.write(normalize(node.getNodeValue()).trim());
			out.write(JavaUtils.LS);
			break;
		}
		case Node.COMMENT_NODE: {
			out.write("<!--");
			out.write(node.getNodeValue());
			out.write("-->");
			out.write(JavaUtils.LS);
			break;
		}
		case Node.PROCESSING_INSTRUCTION_NODE: {
			out.write("<?");
			out.write(node.getNodeName());
			String data = node.getNodeValue();
			if (data != null && data.length() > 0) {
				out.write(' ');
				out.write(data);
			}
			out.write("?>");
			out.write(JavaUtils.LS);
			break;
		}
		}
	}
	
	private static void printPacked(Node node, Writer out) throws IOException {
		if (null == node)
			return;
		switch (node.getNodeType()) {
		case Node.DOCUMENT_NODE: {
			out.write("<?xml version=\"1.0\"?>");
			NodeList nodeList = node.getChildNodes();
			for (int i = 0; i < nodeList.getLength(); i++)
				printPacked(nodeList.item(i), out);
		}
			break;
		case Node.ELEMENT_NODE: {
			out.write('<' + node.getNodeName());
						NamedNodeMap attrs = node.getAttributes();
			for (int i = 0; i < attrs.getLength(); i++) {
				Attr attr = (Attr) attrs.item(i);
				out.write(' ' + attr.getNodeName() + "=\"" + attr.getValue()
						+ '\"');
			}
						NodeList nodeList = node.getChildNodes();
			if (nodeList.getLength() == 0) {
				out.write("/>");
			} else {
				out.write('>');
				for (int i = 0; i < nodeList.getLength(); i++)
					printPacked(nodeList.item(i), out);
				out.write("</" + node.getNodeName() + '>');
			}
		}
			break;
		case Node.ENTITY_REFERENCE_NODE: {
			out.write('&' + node.getNodeName() + ';');
		}
			break;
		case Node.CDATA_SECTION_NODE: {
			out.write("<![CDATA[" + pack(((Text) node).getData()) + "]]>");
		}
			break;
		case Node.TEXT_NODE: {
			out.write(pack(((Text) node).getData()));
		}
			break;
		case Node.COMMENT_NODE: {
			out.write("<!--" + ((CharacterData) node).getData() + "-->");
		}
			break;
		case Node.PROCESSING_INSTRUCTION_NODE: {
			out.write("<?" + node.getNodeName() + ' ' + node.getNodeValue()
					+ "?>");
		}
		}
	}
	
	public static String pack(String s) {
		if (null == s)
			return null;
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			switch (c) {
			case '\n':
				break;
			case '\t':
				break;
			case '\r':
				break;
			default:
				buf.append(c);
			}
		}
		return (buf.toString());
	}
	
	private static String normalize(String s) {
		StringBuffer str = new StringBuffer();
		int len = (s != null) ? s.length() : 0;
		for (int i = 0; i < len; i++) {
			char ch = s.charAt(i);
			switch (ch) {
			case '<': {
				str.append("&lt;");
				break;
			}
			case '>': {
				str.append("&gt;");
				break;
			}
			case '&': {
				str.append("&amp;");
				break;
			}
			case '"': {
				str.append("&quot;");
				break;
			}
			case JavaUtils.NL: {
				if (i > 0) {
					char lastChar = str.charAt(str.length() - 1);
					if (lastChar != JavaUtils.CR) {
						str.append(JavaUtils.LS);
					} else {
						str.append(JavaUtils.NL);
					}
				} else {
					str.append(JavaUtils.LS);
				}
				break;
			}
			default: {
				str.append(ch);
			}
			}
		}
		return (str.toString());
	} 	
	private static void writeIndent(Writer out, int level, String indentString)
			throws IOException {
		if (indentString != null)
			for (int i = 0; i < level; i++)
				out.write(indentString);
	}
	
	public static Document stringToDocument(String docString, boolean IsNSAware) {
		Document doc = null;
		try {
			DocumentBuilder db = null;
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(IsNSAware);
			db = dbf.newDocumentBuilder();
			InputSource is = new InputSource(new StringReader(docString));
			doc = db.parse(is);
		} catch (ParserConfigurationException pcfe) {
			pcfe.printStackTrace();
		} catch (SAXException saxe) {
			saxe.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return doc;
	}
	
	public static Element getElementFromFile(String filename) {
				try {
			DocumentBuilder db = XMLUtils.getDOMBuilder();
			Document doc = null;
						FileInputStream fis = new FileInputStream(filename);
			doc = db.parse(fis);
			return doc.getDocumentElement();
		} catch (ParserConfigurationException pce) {
			pce.printStackTrace();
			return null;
		} catch (SAXException se) {
			se.printStackTrace();
			return null;
		} catch (IOException ioe) {
			ioe.printStackTrace();
			return null;
		}
	}
	
	public static Document parseXMLFile(String filename, boolean validating) {
		try {
						DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			factory.setValidating(validating);
						Document doc = factory.newDocumentBuilder().parse(
					new File(filename));
			return doc;
		} catch (SAXException e) {
					} catch (ParserConfigurationException e) {
		} catch (IOException e) {
		}
		return null;
	}
	
	public static void writeXmlFile(Document doc, String filename) {
		try {
						Source source = new DOMSource(doc);
						File file = new File(filename);
			Result result = new StreamResult(file);
						Transformer xformer = TransformerFactory.newInstance()
					.newTransformer();
			xformer.transform(source, result);
		} catch (TransformerConfigurationException e) {
		} catch (TransformerException e) {
		}
	}
}
